/*
 * ----------------------------------------------------------------------------
 *
 * Gm3u  - A MP3 playlist editor
 *
 * (c) 2000 by Bodo Bauer
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ----------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "id3.h"

typedef struct _ID3TAG {
  gchar *title;
  gchar *artist;
  gchar *album;
  gchar *year;
  gchar *comment;
  gint  track;
  gint  genre;
  gint  revision;
} id3_tag;

static id3_tag *ReadID3Tag ( gchar *file );

static char *typegenre[] = {
   "Blues","Classic Rock","Country","Dance","Disco","Funk","Grunge",
   "Hip-Hop","Jazz","Metal","New Age","Oldies","Other","Pop","R&B",
   "Rap","Reggae","Rock","Techno","Industrial","Alternative","Ska",
   "Death Metal","Pranks","Soundtrack","Euro-Techno","Ambient",
   "Trip-Hop","Vocal","Jazz+Funk","Fusion","Trance","Classical",
   "Instrumental","Acid","House","Game","Sound Clip","Gospel","Noise",
   "Alt. Rock","Bass","Soul","Punk","Space","Meditative",
   "Instrumental Pop","Instrumental Rock","Ethnic","Gothic",
   "Darkwave","Techno-Industrial","Electronic","Pop-Folk","Eurodance",
   "Dream","Southern Rock","Comedy","Cult","Gangsta Rap","Top 40",
   "Christian Rap","Pop/Funk","Jungle","Native American","Cabaret",
   "New Wave","Psychedelic","Rave","Showtunes","Trailer","Lo-Fi",
   "Tribal","Acid Punk","Acid Jazz","Polka","Retro","Musical",
   "Rock & Roll","Hard Rock","Folk","Folk/Rock","National Folk",
   "Swing","Fast-Fusion","Bebob","Latin","Revival","Celtic",
   "Bluegrass","Avantgarde","Gothic Rock","Progressive Rock",
   "Psychedelic Rock","Symphonic Rock","Slow Rock","Big Band",
   "Chorus","Easy Listening","Acoustic","Humour","Speech","Chanson",
   "Opera","Chamber Music","Sonata","Symphony","Booty Bass","Primus",
   "Porn Groove","Satire","Slow Jam","Club","Tango","Samba",
   "Folklore","Ballad","Power Ballad","Rhythmic Soul","Freestyle",
   "Duet","Punk Rock","Drum Solo","A Cappella","Euro-House",
   "Dance Hall","Goa","Drum & Bass","Club-House","Hardcore","Terror",
   "Indie","BritPop","Negerpunk","Polsk Punk","Beat",
   "Christian Gangsta Rap","Heavy Metal","Black Metal","Crossover",
   "Contemporary Christian","Christian Rock","Merengue","Salsa",
   "Thrash Metal","Anime","JPop","Synthpop",NULL
};

/*
 * Display ID3 tag
 */
void DisplayID3Tag ( gchar *file ) {
  id3_tag *tag;
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( ( tag = ReadID3Tag ( file ) ) ) {
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "title_entry" ) ), tag->title );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "artist_entry" ) ), tag->artist );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "album_entry" ) ), tag->album );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "comment_entry" ) ), tag->comment );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "year_entry" ) ), tag->year );
  
    if ( typegenre[tag->genre] ) {
      gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "genre_combo_entry" ) ), typegenre[tag->genre] );
    } else {
      gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "genre_combo_entry" ) ), "??" );
    }
    gnome_appbar_set_status ( status_bar, "" );
  } else {
    gnome_appbar_set_status ( status_bar, "No ID3 tag found" );
    ClearID3Tag();
  }
}

/*
 * Clear ID3 tag
 */
extern void ClearID3Tag ( void ) {
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "title_entry" ) ),   "" );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "artist_entry" ) ),  "" );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "album_entry" ) ),   "" );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "comment_entry" ) ), "" );
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "year_entry" ) ),    "" );  
    gtk_entry_set_text ( GTK_ENTRY ( gtk_object_get_data (GTK_OBJECT (gm3u), "genre_combo_entry" ) ),    "" );  
}


/*
 * Read ID3 Tag from mp3 file
 */
id3_tag *ReadID3Tag ( gchar *file ) {
  int    fd;
  id3_tag *retval = NULL;

  if ( ( fd = open ( file, O_RDONLY ) ) ) {
    gchar buffer[256];

    lseek ( fd,  -128, SEEK_END );
    read ( fd, buffer, 128 );
    close ( fd );

    if ( ! strncmp ( buffer, "TAG", 3 ) ) {
      /* OK, we have an ID3 tag */
      id3_tag *tag = g_malloc0 ( sizeof ( id3_tag ) );

      tag->title  = g_strndup ( buffer+3,  30 );
      tag->artist = g_strndup ( buffer+33, 30 );
      tag->album  = g_strndup ( buffer+63, 30 );
      tag->year   = g_strndup ( buffer+93,  4 );
      
      if ( !buffer[125] && buffer[126] ) {
	tag->revision = 1;
	tag->comment  = g_strndup ( buffer+97, 28 );
	tag->track    = (gint) buffer[126];
	tag->genre    = (gint) buffer[127];	
      } else {
	tag->revision = 0;
	tag->comment  = g_strndup ( buffer+97, 30 );
	tag->track    = 0;
	tag->genre    = (gint) buffer[127];	
      }
      retval = tag;
    }
  }
  return ( retval );
}



