/*
 * ----------------------------------------------------------------------------
 *
 * Gm3u  - A MP3 playlist editor
 *
 * (c) 2000 by Bodo Bauer
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ----------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/*
 *  Pop up the preferences dialog
 */
void on_preferences1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  GtkWidget  *preferences;
  GnomeEntry *root_dir_entry;

  /* create dialog */
  preferences = create_preferences ();

  /* load preferences or use defaults */
  gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "show_id3_tags" ) ), 
				 gnome_config_get_bool ( "gm3u/id3_tags/show=FALSE" ) );
  gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "path_relative" ) ), 
				 gnome_config_get_bool ( "gm3u/path/relative=TRUE" ) );
  gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "path_absolute" ) ), 
				 gnome_config_get_bool ( "gm3u/path/absolute=FALSE" ) );

  root_dir_entry = GNOME_ENTRY ( 
   gnome_file_entry_gnome_entry ( 
    GNOME_FILE_ENTRY ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "root_dir_entry"))));

  gtk_entry_set_text( GTK_ENTRY ( gnome_entry_gtk_entry ( root_dir_entry ) ),  gnome_config_get_string ( "gm3u/path/path=/" ) ); 
  gnome_entry_load_history ( root_dir_entry );

  gtk_widget_set_sensitive (  (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "root_dir_entry" ),
        (gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "path_absolute")))) ? 
	 FALSE : TRUE );

  gtk_entry_set_text( GTK_ENTRY ( gtk_object_get_data ( GTK_OBJECT ( preferences ), "mp3_player_entry")  ),  
                      gnome_config_get_string ( "gm3u/path/mp3_player=xmms" ) ); 
  /* show dialog */
  gtk_widget_show (preferences);
}

/*
 * Cancel button clicked - Close dialog and do nothing else
 */
void on_preferences_cancel_button_button_press_event ( GtkButton *button, gpointer preferences ) {
  gnome_dialog_close ( GNOME_DIALOG ( preferences ) );
}

/* 
 * OK button clicked - Apply preferences and close dialog
 */
void on_preferences_ok_button_button_press_event ( GtkButton *button, gpointer preferences ) {
  on_preferences_apply_button_button_press_event ( button, preferences );
  gnome_dialog_close ( GNOME_DIALOG ( preferences ) );
}

/* 
 * Apply button clicked - Read settings form dialog, save them in config records and sync those to disk
 */
void on_preferences_apply_button_button_press_event ( GtkButton *button, gpointer preferences) {

  /* read entries ... */
  gnome_config_set_bool ( "gm3u/path/absolute", 
	  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON((GtkWidget*) gtk_object_get_data(GTK_OBJECT(preferences), "path_absolute"))));
  gnome_config_set_bool ( "gm3u/path/relative", 
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON((GtkWidget*) gtk_object_get_data(GTK_OBJECT(preferences), "path_relative"))));
  gnome_config_set_bool ( "gm3u/id3_tags/show",  
          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON((GtkWidget*) gtk_object_get_data(GTK_OBJECT(preferences), "show_id3_tags"))));
  gnome_config_set_string ( "gm3u/path/path",  
          gnome_file_entry_get_full_path( (GnomeFileEntry*) gtk_object_get_data(GTK_OBJECT (preferences), "root_dir_entry"), TRUE ));
  gnome_config_set_string ( "gm3u/path/mp3_player",  
                            gtk_entry_get_text ( GTK_ENTRY (  gtk_object_get_data(GTK_OBJECT (preferences), "mp3_player_entry") ) ) );
  /* ... and save them to disk */
  gnome_config_sync ();
}

/*
 *  Set path entry widget sensitive depending on the absolute/relative toggle
 */
void on_path_toggled ( GtkToggleButton *togglebutton, gpointer preferences ) {
  gtk_widget_set_sensitive (  (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "root_dir_entry" ),
        (gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( preferences ), "path_absolute")))) ? 
	 FALSE : TRUE );
}























